#include "PreamplifierDialog.h"
#include "ui_PreamplifierDialog.h"

#include "base/gui_utils.h"

PreamplifierDialog::PreamplifierDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::PreamplifierDialog)
{
    ui->setupUi(this);
    reset_gui();
}

PreamplifierDialog::~PreamplifierDialog()
{
    delete ui;
}

void PreamplifierDialog::reset_gui()
{
    RecursiveWidgetSignalBlocker blocker(this);

    ui->spinBoxBaseline->setValue(config.baseline);
    ui->spinBoxLevel->setValue(config.level);

    if (config.boardSet == AMP_BOARD_AD5622) ui->radioButtonVersion1->setChecked(true);
    if (config.boardSet == AMP_BOARD_AD5622v2) ui->radioButtonVersion2->setChecked(true);
}

void PreamplifierDialog::setPreamplifierConfig(const Ad5622Config &config)
{
    reset_gui();
}

void PreamplifierDialog::on_groupBox_toggled(bool checked)
{
    if(checked) {
        if (ui->radioButtonVersion1->isChecked()) config.boardSet = AMP_BOARD_AD5622;
        if (ui->radioButtonVersion2->isChecked()) config.boardSet = AMP_BOARD_AD5622v2;
    } else {
        config.boardSet = AMP_BOARD_NONE;
    }
    emit preamplifierConfigChanged(config);
}

void PreamplifierDialog::on_radioButtonVersion1_toggled(bool checked)
{
    if(!checked) return;
    config.boardSet = AMP_BOARD_AD5622;
    emit preamplifierConfigChanged(config);
}

void PreamplifierDialog::on_radioButtonVersion2_toggled(bool checked)
{
    if(!checked) return;
    config.boardSet = AMP_BOARD_AD5622v2;
    emit preamplifierConfigChanged(config);
}

void PreamplifierDialog::on_spinBoxBaseline_valueChanged(int baseline)
{
    config.baseline = baseline;
    emit preamplifierConfigChanged(config);
}

void PreamplifierDialog::on_spinBoxLevel_valueChanged(int level)
{
    config.level = level;
    emit preamplifierConfigChanged(config);
}
