//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DOMINODEVICEMANAGER_H
#define DOMINODEVICEMANAGER_H

#include <QMap>
#include <QObject>
#include <QSharedPointer>

#include "base/DeviceIndex.h"

class AdcDeviceProxy;
class AdcStatus;
class DeviceIndexEnabledMap;
class QHostAddress;
struct DeserializersControlBuildParams;

class DominoDeviceManager : public QObject
{
    Q_OBJECT
public:
    explicit DominoDeviceManager(QObject *parent = nullptr);
    ~DominoDeviceManager() override;

signals:
    void statusUpdated(const AdcStatus &);

    void deserializersControlBpUpdated(const DeviceIndex &, const DeserializersControlBuildParams &);

public slots:
    void setHostAddress(const DeviceIndex &index, const QHostAddress &address, int ttl); // remove me
    void setDevices(const DeviceIndexEnabledMap &deviceList);

private:
    void removeDevices(const DeviceIndexSet &setToDel);

    // DeviceDatabase devices;
    QMap<DeviceIndex, QSharedPointer<AdcDeviceProxy> > devices; // access to objects in non-gui thread
};

#endif // DOMINODEVICEMANAGER_H
