/*
**    Copyright 2020 Ivan Filippov, Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DeviceDiscoverMainWindow.h"
#include "base/DeviceId.h"
#include "ui_DeviceDiscoverMainWindow.h"

#include <QApplication>
#include <QCloseEvent>
#include <QDebug>
#include <QHostAddress>
#include <QLabel>
#include <QMessageBox>
#include <QStatusBar>
#include <QString>

#include "device-discover/DiscoverDialog.h"
#include "mldiscover/DeviceDispatcher.h"
#include "mldiscover/LldpDiscovery.h"

DeviceDiscoverMainWindow::DeviceDiscoverMainWindow(QWidget *parent) :
    BaseMainWindow(parent),
    ui(new Ui::DeviceDiscoverMainWindow),
    discoverDialog(new DiscoverDialog(this)),
    deviceDispatcher(new DeviceDispatcher(this)),
    label(new QLabel(this))
{
    ui->setupUi(this);
    setupMenu();

    ui->verticalLayout_1->addWidget(discoverDialog);
    discoverDialog->setSizeGripEnabled(false);
    discoverDialog->show();
    discoverDialog->setAttribute(Qt::WA_Moved, true);

    auto &lldpDiscovery = LldpDiscovery::instance();
    lldpDiscovery.subscribe(deviceDispatcher);
    lldpDiscovery.subscribe(discoverDialog);
    lldpDiscovery.subscribe(this);

    connect(discoverDialog, &QDialog::accepted,
            this, &QMainWindow::close);
    connect(discoverDialog, &QDialog::rejected,
            this, &QMainWindow::close);

    const DeviceIdSet filter({DEVICE_ID_AnyDevice});
    discoverDialog->setDeviceIdFilter(filter);

    ui->statusbar->insertWidget(0, label);
    ui->statusbar->show();
}

DeviceDiscoverMainWindow::~DeviceDiscoverMainWindow()
{
    // blank
}

void DeviceDiscoverMainWindow::closeEvent(QCloseEvent *event)
{
    qDebug() << "Selected devices:" << deviceDispatcher->getDevices();
    event->accept();
}

void DeviceDiscoverMainWindow::lldpCacheUpdated(const LldpInfoCache &new_cache)
{
    QMap<QString, int> models;
    for (auto it = new_cache.begin(); it != new_cache.end(); ++it) {
        models[it->getModelStr()]++;
    }
    QStringList sl;
    for (auto it = models.begin(); it != models.end(); ++it) {
        sl.push_back(QString("%1: %2").arg(it.key()).arg(it.value()));
    }
    // auto s = tr("Discovered: ") + sl.join(", ");
    auto s = QString("Discovered: %1 devices").arg(new_cache.size());
    label->setText(s);
}
