#ifndef FILEWATCHERCORE_H
#define FILEWATCHERCORE_H

#include "lib-common/BaseAppCore.h"

#include "FileWatcherConfig.h"
#include "base/DeviceIndex.h"
#include "base/RcRunId.h"
#include "fsm/FsmState.h"
#include "mstream-lib/MStreamStat.h"
#include "remote-control-server/RemoteControlServerStatus.h"

class FileWatcherConfigDb;
class FileReader;
class DataSender;
struct FileWatcherStartupOptions;
class FileWatcherFsm;
class ConsoleReader;

class FileWatcherCore : public BaseAppCore
{
    Q_OBJECT
public:
    explicit FileWatcherCore(const FileWatcherStartupOptions &so, QObject *parent = nullptr);
    virtual ~FileWatcherCore() override;

signals:
    void configChanged(const FileWatcherConfig &config) const;
    void updateControlState();
    void remoteCompletedStart(bool success, const RcRunId &runId);
    void remoteCompletedStop(bool success, const RcRunId &runId);
    void sendRemoteState(const QString &) const;
    void enableFileMonitor(bool en) const;

public slots:
    void setConfig(const FileWatcherConfig &newConfig);
    void remoteStartRequest(const RcRunId &runId_, const QVariant &startEvNum=0);
    void remoteStopRequest(const RcRunId &runId_, const QVariant &stopEvNum=0);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &status);
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;
    void printStat() const;
    void toggleStatPrint();
    void setFilePath(const QString &p);
    void mStreamStatUpdated(const MStreamStat &ms_stat);

protected:
    bool isRun() const override;
    const BaseFsm *getFsm() const override;

private:
    void initProgDescr();
    void updateProgDescrOptions();
    void sendRunDocument() const;

    const DeviceIndex swDevIndex;
    FileWatcherFsm * const fsm;
    FileWatcherConfigDb * const configDb;
    FileReader * const fileMonitor;
    DataSender * const dataSender;
    ConsoleReader * const consoleReader;

    FileWatcherConfig config;
    RemoteControlServerStatus lastRcStatus;
    MStreamStat lastMStreamStat;
    bool periodicalStatPrint = false;
};

#endif // FILEWATCHERCORE_H
