#include "FileWatcherFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.fileWatcher")
}

FileWatcherFsm::FileWatcherFsm(QObject *parent)
    : BaseFsm{parent}
{
    setObjectName(__func__);
    recreateInput(new FileWatcherFsmInput());
}

bool FileWatcherFsm::isTransitionAllowed(const FsmState &toState) const
{
    static const QMap<QPair<FsmState, FsmState>, bool> map
        = {
            {{FsmState::Ready, FsmState::Run}, true},
            {{FsmState::Run, FsmState::Ready}, true},
        };
    return map.value({getState(), toState}, false);
}

void FileWatcherFsm::setFileValid(bool valid)
{
    auto i = getInput();
    if(i->fileValid == valid)
        return;
    i->fileValid = valid;
    evaluate();
}

bool FileWatcherFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    auto i = getInput();
    if(getState() == FsmState::Error) {
        if (i->fileValid) {
            setState(FsmState::Ready);
            return true;
        }
    } else {
        if(!i->fileValid) {
            qDebug(fsm) << __PRETTY_FUNCTION__ << "Go to Error state (file is missing or empty)";
            setState(FsmState::Error);
            return true;
        }
    }

    return false;
}

void FileWatcherFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

QDebug operator<<(QDebug dbg, const FileWatcherFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "fileValid:" << in.fileValid << "; ";
    return dbg;
}
