#ifndef TTBMODEL_H
#define TTBMODEL_H

#include <QAbstractTableModel>

class ConfigKey;
class DeviceIndex;
struct LldpInfo;
class LldpInfoCache;
class ModularDeviceConfig;
struct TtbStatus;
class TtbMetaTable;
struct HrbAppConfig;

class TtbModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    TtbModel(QObject *parent = nullptr);
    ~TtbModel();
    void setEnabled(bool en);

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    Qt::ItemFlags flags(const QModelIndex & index) const override;
    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role) override;

signals:
    void ttbDeviceConfigChanged(const ModularDeviceConfig &config);
    void configChanged(const HrbAppConfig &) const;

public slots:
    void lldpCacheUpdated(const LldpInfoCache &lldp);
    void setDevConfig(const ModularDeviceConfig &config);
    void deviceStatusUpdated(const TtbStatus &status);
    void addDevice(const ModularDeviceConfig &cfg);
    void removeDevice(const DeviceIndex &index);
    void setConfig(const ConfigKey &key, const HrbAppConfig &cfg);

private:
    inline static bool isChCol(int col);
    void invalidateRow(const DeviceIndex &index);
    void sendDataChanged(QVector<int> roles = QVector<int>());
    QVariant data_BackgroundRole(const DeviceIndex &index, int column) const;
    QVariant data_CheckStateRole(const DeviceIndex &index, int column) const;
    QVariant data_DisplayRole(const DeviceIndex &index, int column) const;

    TtbMetaTable * const devices;
    HrbAppConfig * const appCfg;
    bool showAveragedBusy = true;
    bool enabled = true;
};

#endif // TTBMODEL_H
