#ifndef MSCHISTMODEL_H
#define MSCHISTMODEL_H

#include <QAbstractItemModel>

#include "MscHistColumns.h"
#include "device-modules/MscCycleCounters/MscCycleStatus.h"
#include "lib-common/BaseDeviceAppConfig.h"

class ConfigKey;
class DeviceIndex;
struct MscModelItem;
struct MscBuildParams;

class MscHistModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    MscHistModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(MscHistColumns::COL_TOTAL);
    }

public slots:
    void cycleStatusUpdated(const DeviceIndex &devIndex, const MscCycleStatus &newSt);

signals: void configChanged(const BaseDeviceAppConfig &config) const;
public slots: void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);

private:
    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant foregroundRoleData(const QModelIndex &index) const;
    inline MscModelItem *getItemByIndex(const QModelIndex &index) const;
    inline MscHistColumns getColumnByIndex(const QModelIndex &index) const;
    void columnDataChanged(MscHistColumns col);
    void reset();

    MscModelItem * const rootItem;
    BaseDeviceAppConfig config;
    QMap<DeviceIndex, MscCycleStatus> status;
    QMap<DeviceIndex, int> histNum;
};

#endif // MSCHISTMODEL_H
