#include "MscThresholdView.h"
#include "MscSpinBoxDelegate.h"

#include <QDebug>

#include "MscThresholdColumns.h"

class MscThresholdDelegate : public MscSpinBoxDelegate
{
public:
    MscThresholdDelegate(QObject *parent = 0) : MscSpinBoxDelegate(parent) {}

protected:
    SpinBoxParams getParams(int col) const override
    {
        if(col != static_cast<int>(MscThresholdColumns::COL_THRESHOLD))
            Q_UNREACHABLE();

        static SpinBoxParams p{.step=10,.max=1250,.min=-1250,.suffix=" mV"};
        return p;
    }
};

MscThresholdView::MscThresholdView(QWidget *parent)
    : QTreeView(parent)
    , delegate(new MscThresholdDelegate(this))
{}

void MscThresholdView::setModel(QAbstractItemModel *model)
{
    QTreeView::setModel(model);
    setItemDelegateForColumn(static_cast<int>(MscThresholdColumns::COL_THRESHOLD),
                             delegate);

    connect(model, &QAbstractItemModel::modelReset,
            this, &MscThresholdView::onModelReset);
}

void MscThresholdView::onModelReset()
{
    addDelegate();
}

void MscThresholdView::addDelegate(const QModelIndex &parent)
{
    const auto cols = {static_cast<int>(MscThresholdColumns::COL_THRESHOLD)};
    if(parent.isValid()) {
        for(auto col : cols) {
            const auto &i = parent.siblingAtColumn(col);
            if(!isPersistentEditorOpen(i))
                openPersistentEditor(i);
        }
    }

    const auto &m = model();
    int r1 = m->rowCount(parent);
    for(int i=0; i<r1; ++i) {
        auto chIndex = m->index(i, 0, parent);
        addDelegate(chIndex);
    }
}
