//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TRIGMAINFORM_H
#define TRIGMAINFORM_H

#include <QWidget>

#include "RcConfig.h"
#include "fsm/FsmState.h"

struct FirmwareBuildParams;
class RcConfigDispatcher;
struct SpillPlotData;
class TrigChannelConfig;
struct TrigChannelState;
struct TrigStatus;
struct TriggerHistogramData;

namespace Ui {
class TrigMainForm;
}

class TrigMainForm : public QWidget
{
    Q_OBJECT

public:
    explicit TrigMainForm(QWidget *parent = nullptr);
    ~TrigMainForm();
    void setConfigDispatcher(RcConfigDispatcher *p);
    void updateExtTrigGroup();
    void clearStats();
    static QString timeStr(quint64 t);
    static double timeFromClocks(quint64 t);
    void updateRemoteBlocking(bool remoteConnected);

signals:
    void configChanged(const RcConfig &config);
//    void updateLeds(quint16 curState, quint16 trig, quint16 xOff, QVector<QString> chTime);
    void updateChState(const TrigChannelState &);
    void clearTrigStats(bool onStartRun);
    void resetAll();

    // proxy signals
    void spillPlotUpdated(const SpillPlotData &);
    void triggerHistogramUpdated(const TriggerHistogramData &);
    void rcConfigUpdated();
    void rcFsmStateChanged(FsmState st) const;

public slots:
    void setConfig(const RcConfig &newConfig);
    void setFirmwareBuildParams(const FirmwareBuildParams &fbp);
    void trigStatusUpdated(const TrigStatus &status);
    void spillPlotStepUpdated();
    void setRcRunId(const RcRunId &id);

private slots:
    void on_radioButtonTrigOpen_toggled(bool open);
    void configUpdated(quint16 xOff);
    void on_pushButtonClearStat_clicked();
    void on_pushButtonResetAll_clicked();
    // Trig Input
    void on_doubleSpinBoxTrigTimerFreq_valueChanged(double arg1);
    void on_doubleSpinBoxTrigRandomFreq_valueChanged(double arg1);
    void on_doubleSpinBoxTrigRandomDeadTime_valueChanged(double arg1);
    void on_checkBoxTrigIn_toggled(bool checked);
    void on_checkBoxTrigIn2_toggled(bool checked);
    void on_checkBoxTrigIn3_toggled(bool checked);
    void on_checkBoxTrigIn4_toggled(bool checked);
    void on_checkBoxTrigLvds_toggled(bool checked);
    void on_checkBoxTrigPeriod_toggled(bool checked);
    void on_checkBoxTrigRandom_toggled(bool checked);

//    void on_checkBoxSpillSourceAuto_toggled(bool);

    void onRcFsmStateChange(FsmState st);

private:
    void sendTrigStatusMetrics(const TrigStatus &prevTrigStatus, const TrigStatus &status);
    void setTrigSrcMask(quint16 mask, bool checked);
    void updateTableSize();
    void restoreGuiFromConfig();

    Ui::TrigMainForm *ui;
    RcConfigDispatcher * configDispatcher = nullptr;
    RcConfig * const config;
    RcRunId rcRunId;
    TrigChannelConfig * trigChannelConfig;
    TrigStatus * const prevTrigStatus;
};

#endif // TRIGMAINFORM_H
