/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VMETRIGSCONTROLLER_H
#define VMETRIGSCONTROLLER_H

#include <QObject>

#include "AbstractDeviceController.h"
#include "base/DeviceIndexEnabled.h"
#include "base/RcRunId.h"
#include "fsm/FsmState.h"
#include "pnp-server/PNPMessage.h"

class ConfigKey;
class DeviceDispatcher;
class RcConfigDispatcher;
struct RcConfig;
class MessageTreeItem;
struct VmeTrigConfig;
struct VmeTrigStatus;
class VmeTrigsControllerPrivate;

class VmeTrigsController : public AbstractDeviceController
{
    Q_OBJECT
public:
    explicit VmeTrigsController(QObject *parent = nullptr);
    ~VmeTrigsController() override;
    QSet<DeviceId> getDeiveIdFilter() override;

signals:
    void stateChanged(const FsmState &);
//    void deleteTrig(const DeviceIndex &serial); // serial==0 - for all
    void trigConfigChanged(const DeviceIndex &index, const VmeTrigConfig &config);
    void vmeReadoutUpdated(const DeviceIndex &serial, bool evNumReadReset);
//    void runError(const RcRunId &runId, QString errStr);
    void sendSlaveMessage(const MessageTreeItem &);
    void vmeMapUpdated(QMap<DeviceIndex, QString> map) const;
    void evNumRequest(int checkIndex) const;
    void evNumResponse(int checkIndex, const DeviceIndex &index, quint64 evNum) const;

    // required for model
    void deviceAdded(const DeviceIndex &index);
    void deviceRemoved(const DeviceIndex &index);
    void deviceStatusUpdated(const DeviceIndex &index, const VmeTrigStatus &status);
    void printFsmInput();

public slots:
    void setConfig(const RcConfig &newConfig) override;
    void setDeviceList(const DeviceIndexEnabledMap &devices) override;
    void setTrigConfig(const DeviceIndex &index, const VmeTrigConfig &config);
    void onRunStart(const RcRunId &runId);
    void onRunStop();
    void pnpCacheUpdated(const PnpInfoCache &cache);

private:
    Q_DECLARE_PRIVATE(VmeTrigsController)
    VmeTrigsControllerPrivate * const d_ptr;
};

#endif // VMETRIGSCONTROLLER_H
