#ifndef TRCDEVICE_H
#define TRCDEVICE_H

#include "ModularDeviceConfig.h"
#include "ModularMregDevice.h"
#include "RcCounters/RcCounters.h"
#include "TrcStatus.h"

class TrcErcModule;
class LegacyDeviceModule;
class PhysicalTriggerModule;

class TrcDevice : public ModularMregDevice
{
    Q_OBJECT
public:
    explicit TrcDevice(const DeviceIndex &index, QObject *parent = nullptr);

signals:
    void statusUpdated(const TrcStatus &) const;
    void countersUpdated(BaseDevCountersMap) const;
    void deviceInfoUpdated(DeviceIndex index, quint16 auxNch, quint16 phyNch) const;

public slots:
    void preRunReset(const QVariant &startEvNum);
    void resetAll();
    void softClear();

protected slots:
    virtual void on_deviceConnected() override;
    virtual void on_deviceDisconnected() override;

protected:
    bool readStatus() override;
    bool writeConfig() override;
    virtual void createModules() override;

private:
    bool updateCounters();
    bool readTrcStatus();
    void clearStatistics();
    void StartRun();
    void writeTime();
    quint16 getDeviceCtrlReg() const;

protected:

    TrcStatus status;

    QSharedPointer<LegacyDeviceModule> legacyDevice;
    QSharedPointer<PhysicalTriggerModule> phyTrigMod;
    QSharedPointer<TrcErcModule> evReadoutMod;
};

#endif // TRCDEVICE_H
