#ifndef TTVXSDEVICE_H
#define TTVXSDEVICE_H

#include <QObject>
#include <QMetaType>

#include "ModularMregDevice.h"
#include "TtvxsConfig.h"
#include "TtvxsStatus.h"
#include "base/DeviceIndex.h"

class LegacyDeviceModule;
struct FeLinkStatus;
class RcRunId;

namespace mlink {
class RegOpVector;
}

class TtvxsDevice : public ModularMregDevice
{
    Q_OBJECT
public:
    enum NetworkPortType {SFP0, SFP1, SFP2, SFP3};
    explicit TtvxsDevice(const DeviceIndex &devIndex, QObject *parent = nullptr);

signals:
    void statusUpdated(const TtvxsStatus &status);
    void topEvNumResponse(int check_index, quint64 evNum);

public slots:
    bool evNumReset(const DeviceIndex &index = {}, const QVariant &startEvNum = {});
    bool getTrigEvNum(int check_index, bool top_level);
    void onRestartAll(const RcRunId &runId, const QVariant &startEvNum);
    bool softClear(const DeviceIndex &i);
    bool clearStat(DeviceIndex i);
    bool resetTimestamp(const DeviceIndex &i = {});

protected slots:
    void on_deviceConnected() override;
    void on_deviceDisconnected() override;

protected:
    bool readStatus() override;
    bool writeConfig() override;

private:
    bool writeTrigCtrlStatConfig(const TrigCtrlStatConfig &cfg);
    bool writeTtvxsCoreConfig(const TtvxsCoreConfig &cfg);
    bool readTtvxsBuildParams();
    std::optional<TtvxsDeviceCoreStatus> readTtvxsDeviceCoreStatus();
    bool readTtvxsStatus();
    mlink::RegOpVector writeCoreConfigCommand(const TtvxsCoreConfig &cfg) const;
    static QString networkPortName(int portIndex);
    reg_base_ctrl currentBaseCtrl() const;
    reg_rc_ctrl currentRegRcCtrl() const;

    TtvxsStatus status;
    // reg_rc_ctrl rcCtrl = {};

    QSharedPointer<LegacyDeviceModule> legacyDevice;
};

#endif // TTVXSDEVICE_H
