#ifndef CONFIGDOCUMENT_H
#define CONFIGDOCUMENT_H

#include <QMetaType>

#include "ConfigOid.h"
#include "ConfigSelector.h"
#include "ConfigMeta.h"
#include <QJsonObject>

class ConfigDocument
{
public:
    friend class ConfigDocumentConverter;
    ConfigDocument() = default;
    ConfigDocument(const ConfigSelector &_cs)
        : cs(_cs)
    {}
    ConfigDocument(const QJsonObject &_config)
        : config(_config)
    {}
    ConfigDocument(const ConfigOid &_oid,
                   const ConfigSelector &_cs,
                   const ConfigMeta &_cm)
        : oid(_oid)
        , cs(_cs)
        , cm(_cm)
    {}
    ConfigDocument(const ConfigOid &_oid,
                   const ConfigSelector &_cs,
                   const ConfigMeta &_cm,
                   const QJsonObject &_config)
        : oid(_oid)
        , cs(_cs)
        , cm(_cm)
        , config(_config)
    {}
    bool isEmpty() const { return cs.isEmpty();}
    // unused
    // void setConfigOid(const ConfigOid &_oid) {oid = _oid;}
    void setConfigSelector(const ConfigSelector &_cs) {cs = _cs;}
    void setConfigMeta(const ConfigMeta &_cm) {cm = _cm;}
    void setConfig(const QJsonObject &_config) { config = _config; }
    ConfigOid getConfigOid() const { return oid; }
    ConfigSelector getConfigSelector() const { return cs; }
    ConfigMeta getConfigMeta() const { return cm; }
    QJsonObject getConfig() const { return config; }

    QString toString() const;

    friend bool operator==(const ConfigDocument &l, const ConfigDocument &r) = default;

private:
    ConfigOid oid;
    ConfigSelector cs;
    ConfigMeta cm;
    QJsonObject config;
};

Q_DECLARE_METATYPE(ConfigDocument)

#endif // CONFIGDOCUMENT_H
