/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ProgramIndexConverter.h"

namespace {
constexpr const char *k_domain = "domain";
constexpr const char *k_index = "index";
}

template<>
bool AbstractConfigConverter<ProgramIndex>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ProgramIndex>::toJsonObject(const ProgramIndex &in)
{
    QVariantMap m;
    m[k_domain] = QVariant::fromValue(in.getDomain());
    m[k_index] = QVariant::fromValue(in.getIndex());
    return fromVariantMap(m);
}

template<>
ProgramIndex AbstractConfigConverter<ProgramIndex>::fromJsonObject(const QJsonObject &m)
{
    ProgramIndex out;
    auto domain = QVariant::fromValue(m[k_domain]).value<QString>();
    auto index = QVariant::fromValue(m[k_index]).value<QString>();
    return ProgramIndex(domain, index);
}

template<>
QJsonValue AbstractConfigConverter<ProgramIndex>::toJsonValue(const ProgramIndex &in)
{
    return toJsonObject(in);
}

template<>
ProgramIndex AbstractConfigConverter<ProgramIndex>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ProgramIndex>::registerSubTypes()
{
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    if (QMetaType::hasRegisteredComparators<ProgramIndex>())
        return;
    qRegisterMetaTypeStreamOperators<ProgramIndex>();
    QMetaType::registerComparators<ProgramIndex>();
#endif

//    QMetaType::registerConverter<ProgramIndex, QString>(&ProgramIndex::toString);
//    QMetaType::registerConverter<QString, ProgramIndex>(&ProgramIndex::fromString);
}
