#ifndef TAITIME_H
#define TAITIME_H

#include <QMetaType>

struct TaiTime
{
    friend bool operator==(const TaiTime &, const TaiTime &) = default;
    TaiTime &operator-=(const TaiTime &r)
    {
        qint64 raw = toNs() - r.toNs();
        sec = raw / ns_factor;
        ns = raw % ns_factor;
        valid &= r.valid;
        return *this;
    }
    friend TaiTime operator-(TaiTime l, const TaiTime &r)
    {
        l -= r;
        return l;
    }

    bool isNull() const { return !ns && !sec; }
    bool isValid() const { return valid && !isNull(); }
    void addNs(int64_t _ns)
    {
        _ns += ns;
        sec += _ns / ns_factor;
        ns = _ns % ns_factor;
    }
    qint64 toNs() const { return qint64(sec) * ns_factor + ns; }
    QString toString() const;

    int32_t sec = 0;
    int32_t ns = 0;
    bool valid = false;

private:
    static inline constexpr const qint64 ns_factor = 1'000'000'000;
};

Q_DECLARE_METATYPE(TaiTime)

QDebug operator<<(QDebug stream, const TaiTime &tai);

#endif // TAITIME_H
