/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DAQBASEAPP_H
#define DAQBASEAPP_H

#include "base/DaqModule.h"
#include "base/ConfigSelector.h"

#include <QCommandLineOption>
#include <QList>
#include <QSharedPointer>

class QCoreApplication;

class QCommandLineParser;

class DaqBaseApp
{
public:
    DaqBaseApp(int &_argc, char *_argv[]);
    virtual ~DaqBaseApp() = default;

    bool hasConfigListOpt() const { return option_config_list; }
    virtual void init();

protected:
    void parseOptions();
    virtual void setupOptions();
    virtual void processParseResults(QCommandLineParser &parser);
    virtual QCoreApplication *createApplication() const;
    virtual int exec();
    virtual bool hasConfigSelectorOption() const;

    int &argc;
    char **argv;

    QList<QCommandLineOption> options;
    bool option_no_syslog = false;
    bool option_debug = false;
    bool option_config_list = false;

private:
    Q_DISABLE_COPY(DaqBaseApp);
    void showVersion() const;

    QSharedPointer<QCoreApplication> app;

    QCommandLineOption nosyslogOption;
    QCommandLineOption debugOption;
    QCommandLineOption commonCfgFileOption;
    QCommandLineOption configNameOption;
    QCommandLineOption configListOption;
    QCommandLineOption versionOption;
    QCommandLineOption helpOption;
};

#endif // DAQBASEAPP_H
