/*
**    Copyright 2020-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FW_VERSION_H
#define FW_VERSION_H

#include <cstdint>

class QString;

struct fw_version_t
{
    constexpr fw_version_t()
        : v_maj(0), v_min(0), revision(0) {}

    constexpr fw_version_t(uint8_t fw_maj, uint8_t fw_min, uint16_t fw_rev)
        : v_maj(fw_maj), v_min(fw_min), revision(fw_rev) {}
    constexpr fw_version_t(uint16_t fw_ver, uint16_t fw_rev)
        : v_maj((fw_ver >> 8) & 0xFF), v_min(fw_ver & 0xFF), revision(fw_rev) {}
    bool isNull() const { return !v_maj && !v_min && !revision; }

    QString toString() const;
    static QString toString(const fw_version_t &r);
    static fw_version_t fromString(const QString &r);

    uint16_t v_maj;
    uint16_t v_min;
    uint16_t revision;
};

inline bool operator==(const fw_version_t &lhs, const fw_version_t &rhs) {
    return lhs.v_maj == rhs.v_maj &&
            lhs.v_min == rhs.v_min &&
            lhs.revision == rhs.revision;
}
inline bool operator<(const fw_version_t &lhs, const fw_version_t &rhs) {
    if (lhs.v_maj < rhs.v_maj)
        return true;
    if (lhs.v_maj == rhs.v_maj && lhs.v_min < rhs.v_min)
        return true;
    if (lhs.v_maj == rhs.v_maj && lhs.v_min == rhs.v_min && lhs.revision < rhs.revision)
        return true;
    return false;
}
inline bool operator> (const fw_version_t& lhs, const fw_version_t& rhs){ return rhs < lhs; }
inline bool operator<=(const fw_version_t& lhs, const fw_version_t& rhs){ return !(lhs > rhs); }
inline bool operator>=(const fw_version_t& lhs, const fw_version_t& rhs){ return !(lhs < rhs); }

#endif // FW_VERSION_H
