/*
**    Copyright 2009-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DSPLIB_BRUTEFT_H
#define DSPLIB_BRUTEFT_H

#include <vector>

namespace dsplib {

class BruteFT
{
public:
    // Discrete Fourier Transform, real form, order N
    static void DFT (const std::vector<double> &x, std::vector<double> *S, std::vector<double> *C);
    static void DFT_tmp (const std::vector<double> &x, std::vector<double> *S, std::vector<double> *C);
    //  void DFT (const std::vector<double> &x, std::vector<double> *S, std::vector<double> *C);

    // Inverse Discrete Fourier Transform, real form, order N
    static void IDFT (std::vector<double> *x, const std::vector<double> &S, const std::vector<double> &C);
    static std::vector<double> IDFT (const std::vector<double> &S, const std::vector<double> &C);

    // Calculate power from rectangular FFT
    static void RECT2POW (const std::vector<double> &S, const std::vector<double> &C, std::vector<double> *out);
    static std::vector<double> RECT2POW (const std::vector<double> &S, const std::vector<double> &C);

    // Calculate phase from rectangular FFT
    static std::vector<double> RECT2PHA (const std::vector<double> &S, const std::vector<double> &C);
};

} // namespace dsplib

#endif // DSPLIB_BRUTEFT_H
