/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef BASEDEVICETABLEMODEL_H
#define BASEDEVICETABLEMODEL_H

#include "DeviceTable.h"
#include "base/DeviceIndexEnabled.h"
#include "mvc/DeviceIndexTableModel.h"

class BaseDeviceTableModel : public DeviceIndexTableModel
{
    Q_OBJECT

public:
    enum Columns {
        COL_ENABLE,
        COL_ONLINE,
        COL_DYNAMIC,
        COL_DEVICE_INDEX,
        COL_DEVICE_ID,
        COL_SERIAL,
        COL_FIRM_WARE,
        COL_SLOT,
        COL_IP,
        COL_MAC,
        COL_MASTER,
        COL_M_STREAM,
        COL_TOTAL
    };

    explicit BaseDeviceTableModel(QObject *parent = nullptr)
        : DeviceIndexTableModel(parent) {}
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    //void setDevices(const DeviceTable &list);
    void addStaticDevices(const DeviceTable &list);
    void setDynamicFlag(const DeviceIndex &index, DeviceTableEntry::DynamicFlag flag);
    //DeviceIndexEnabledMap getStaticDevices() const;
    DeviceTable getDevices() const;
    DeviceIndexSet getDeviceIndices() const;
    DeviceIndex deviceIndexOf(int row) const override;
    int rowOf(const DeviceIndex &sourceIndex) const override;

    QMap<int, bool> defaultColumnState() const override;

protected:
    void addStaticDevice(const DeviceTableEntry &index);
    void invalidateRow(const DeviceIndex &index);

    DeviceTable devices;
};

#endif // BASEDEVICETABLEMODEL_H
