/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef LLDPDISCOVERYPRIVATE_H
#define LLDPDISCOVERYPRIVATE_H

#include <QElapsedTimer>
#include <QMap>
#include <QObject>

#include "LldpDiscovery.h"
#include "base/DeviceIndex.h"

class MldpListener;
class QTimer;

class LldpDiscoveryPrivate : public QObject
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(LldpDiscovery)
public:
    explicit LldpDiscoveryPrivate(QObject *parent = nullptr);

signals:
    void cacheUpdated(const LldpInfoCache &);
    void deviceStateChanged(const DeviceStatePair &);
    void setDeviceIdFilter(const DeviceIdSet &filter);

private slots:
    void receivedLldpInfo(const LldpInfo &lldpInfo);
    void validateCache();
    void sendUpdates();
    void throttleTimeout();

private:
    void requestUpdate();

private:
    LldpDiscovery * q_ptr;
    MldpListener * const mldpListener;
    QTimer * const cacheValidateTimer;
    QTimer * const throttleTimer;
    QMap<DeviceIndex, LldpInfo> map;
    bool mapModified = false;
    QElapsedTimer updateTime;
};

#endif // LLDPDISCOVERYPRIVATE_H
