#include "Ad5328Module.h"

#include "Ad5328/Ad5328Config.h"
#include "base/DeviceId.h"
#include "device-modules/MregBusInterface.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

enum {
    REG_DAC_CTRL_BASE = 0x0, // for channels 0..7
//    REG_DAC_CTRL_DAC2 = 0x1  // for channels 8..15
};

enum {
    DAC_CMD_RESET           = 0xF000,
    DAC_CMD_GAIN            = 0x8000,
    DAC_CMD_SINGLE_UPDATE   = 0xA002
};


Ad5328Module::Ad5328Module(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
    if (!isAvailable()) {
        if  (getDeviceId() == DEVICE_ID_PHOS_FEC)
            setBaseAddr(bus.baseAddr(), 0xFF);
    }
}

QList<AFI_SDB_ID> Ad5328Module::getSupportedSdbIds()
{
    return {AFI_SDB_AD5328_x2_SPI};
}

bool Ad5328Module::writeConfig(const Ad5328Config &config)
{
    RegOpVector r;
    for (auto chipIt = config.chips.begin(); chipIt != config.chips.end(); ++chipIt) {
        auto chip = chipIt.key();
        const auto &chipConfig = chipIt.value();
        quint16 gainCmd = DAC_CMD_GAIN; // default GAIN control
        //        gainCmd |= 0x03; // set Vdd as ref.
        if (chipConfig.bufferedInput)
            gainCmd |= 0x0C; // set Buffered input
        if (chipConfig.doubleGain)
            gainCmd |= 0x30; // set double GAIN
        r.RegWrite16(REG_DAC_CTRL_BASE + chip, gainCmd);
        for (auto outIt = chipConfig.dacValue.begin(); outIt != chipConfig.dacValue.end(); ++outIt) {
            const quint16 ch = outIt.key();
            quint16 dacData = std::clamp(outIt.value(), 0, 0xFFF);
            quint16 code = (ch << 12) | dacData;
            r.RegWrite16(REG_DAC_CTRL_BASE + chip, code);
            // qDebug() << __func__ << chip << ch << dacData;
        }
        r.RegWrite16(REG_DAC_CTRL_BASE + chip, DAC_CMD_SINGLE_UPDATE);
    }
    return regOpExecRebased(r);
}
