#include "DlnpCalibrationTriggerModule.h"

#include "regio/regop.h"

#include "DlnpCalibrationTriggerConfig.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID            = 0x0,
    REG_CONTROL         = 0x2,
    REG32_CLOCK_FREQ    = 0x6,

    REG32_SPILL_OFFSET  = 0x8,
    REG32_TRIG_PERIOD   = 0xA,
    REG_TRIG_COUNT      = 0xC,
};
constexpr const quint16 CTRL_BIT_ENABLE = 1;
}

QList<AFI_SDB_ID> DlnpCalibrationTriggerModule::getSupportedSdbIds()
{
    return { AFI_SDB_Calibration_Trigger };
}

void DlnpCalibrationTriggerModule::post_init()
{
    buildParams.reset();
    DlnpCalibrationTriggerBuildParams bp;
    RegOpVector r;
    r.RegRead32(REG32_ID, &bp.id);
    r.RegRead32(REG32_CLOCK_FREQ, &bp.clockKHz);
    if (regOpExecRebased(r)) {
        buildParams = bp;
    }
}

bool DlnpCalibrationTriggerModule::writeConfig(const DlnpCalibrationTriggerConfig &cfg)
{
    if (!buildParams || !buildParams->isValid())
        return {};

    RegOpVector r;
    r.RegWrite16(REG_CONTROL, cfg.en ? CTRL_BIT_ENABLE : 0);
    r.RegWrite32(REG32_SPILL_OFFSET, cfg.offset);
    r.RegWrite32(REG32_TRIG_PERIOD, cfg.period);
    r.RegWrite16(REG_TRIG_COUNT, cfg.count);
    return regOpExecRebased(r);
}
