/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MREGBUSINTERFACE_H
#define MREGBUSINTERFACE_H

#include <QtCore>

#include "sdb/SdbDevice.h"

class SdbModuleIndex;
class QtMregDevice;
enum AFI_SDB_ID : uint32_t;

struct AddrRange
{
    AddrRange() = default;
    AddrRange(QPair<quint16, quint16> pair)
        : addr_first(pair.first), addr_last(pair.second) {}
    AddrRange(quint16 addr_first, quint16 addr_last)
        : addr_first(addr_first), addr_last(addr_last) {}
    bool isNull() const { return addr_last == 0; }
    quint16 lastAddr() const { return addr_last; }

    quint16 addr_first = 0;
    quint16 addr_last = 0;
};

struct MregBusInterface
{
    MregBusInterface(QtMregDevice *qtMregDevice, const SdbDevice &sdbDevice = {})
        : dev(qtMregDevice), sdbDevice(sdbDevice) {
        // SDB address is counted in bytes, divide by 2
        setAddrRange(AddrRange(sdbDevice.component.addr_first>>1,
                               sdbDevice.component.addr_last>>1));
    }
    void clear() {sdbDevice = {}; addrRange = {}; }
    void setSdbDevice(const SdbDevice &r) {sdbDevice = r; }
    void setAddrRange(const AddrRange &range) { addrRange = range; }
    quint16 baseAddr() const { return addrRange.addr_first; }
    quint16 lastAddr() const { return addrRange.lastAddr(); }
    bool isAvailable() const { return !addrRange.isNull(); }
    SdbModuleIndex moduleIndex() const;
    AFI_SDB_ID getSdbId() const;

    QtMregDevice * const dev;
    SdbDevice sdbDevice = {};
private:
    AddrRange addrRange; // RegIO reg_addr
};
#endif // MREGBUSINTERFACE_H
