#ifndef MSCINPUTSTATUS_H
#define MSCINPUTSTATUS_H

#include <cstdint>
#include <QVector>

#include "base/TaiTime.h"

struct MscInputBuildParams
{
    friend bool operator==(const MscInputBuildParams &, const MscInputBuildParams &) = default;

    uint16_t channels = 0; // Number of input hit channels
    uint16_t hists = 0; // Number of histogramers
    uint16_t ceInputs = 0; // Number of external CE inputs
};

struct MscInputStatus
{
    using CeInputEdgeTime = QPair<TaiTime, TaiTime>;
    using CeInputEdgeTimeRaw = QPair<quint64, quint64>;

    friend bool operator==(const MscInputStatus &, const MscInputStatus &) = default;
    void clear() { *this = {}; }
    bool ceInputState(int channel) const;
    QVector<bool> ceInputStates() const;
    CeInputEdgeTime ceInputEdgeTime(int channel) const;

    int ceInputCount = 0;
    quint16 ceInputStateRaw = 0; // Current state of external CE (TTL)
    uint16_t hitState = 0; // Current state of input HITs
    QVector<CeInputEdgeTimeRaw> ceInputEdgeTimesRaw;
    // QVector<QPair<TaiTime, TaiTime>> ceInputEdgeTimes; // [ceId] = {rise, fall}
};

#endif // MSCINPUTSTATUS_H
