#include "NetworkPortModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

#include "NetworkPortModulePrivate.h"

NetworkPortModule::NetworkPortModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
    , d_ptr(new NetworkPortModulePrivate(this))
{}

NetworkPortModule::~NetworkPortModule()
{
    delete d_ptr;
}

void NetworkPortModule::post_init()
{
    Q_D(NetworkPortModule);

    d->descr = {};
    switch (majorVersion()) {
    case 1:
    {
        d->descr = d->getDescription_v1();
        break;
    }
    case 2:
    {
        if (!isAvailable())
            return;

        auto regOp = d->getRegOp<NetworkPortDescription>();
        if (!regOpExecRebased(regOp))
            return;

        d->descr = d->getDescription(regOp);
    }
    default: break;
    }
}

QList<AFI_SDB_ID> NetworkPortModule::getSupportedSdbIds()
{
    return {AFI_SDB_ETH_PORT_V1};
}

std::optional<NetworkPortStatus> NetworkPortModule::readStatus()
{
    Q_D(const NetworkPortModule);
    if (!isAvailable())
        return {};

    auto regOp = d->getRegOp<NetworkPortStatus>();
    if (!regOpExecRebased(regOp))
        return {};

    const auto &status = d->getStatus(regOp);
    return status;
}

bool NetworkPortModule::setConfig(const NetworkPortConfig &config)
{
    Q_D(NetworkPortModule);
    return d->setConfig(config);
}
