/*
**    Copyright 2014-2016 Ivan Filippov
**    Copyright 2014-2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "SpillConfig.h"

#include <QDataStream>
#include <QDebug>

QDebug operator <<(QDebug stream, const SpillConfig &spill)
{
    stream.noquote().nospace()
            << "SpillConfig("
            << "phase 1: " << spill.durPhase1
            << ", phase 2: " << spill.durPhase2
            << ", phase 3: " << spill.durPhase3
            << ", phase 4: " << spill.durPhase4
            << ", phase 5: " << spill.durPhase5
            << ", source select: " << spill.source
            << ", softSpill: " << spill.softSpill
            << ", timer period: " << spill.timerPeriod
            << ")";
    return stream;
}

QDataStream &operator<<(QDataStream &out, const SpillConfig &r)
{
    out
            << r.durPhase1
            << r.durPhase2
            << r.durPhase3
            << r.durPhase4
            << r.durPhase5
            << static_cast<int>(r.source)
            << r.softSpill
            << r.timerPeriod
               ;
    return out;
}

QDataStream &operator>>(QDataStream &in, SpillConfig &r)
{
    int enumVal;
    in
            >> r.durPhase1
            >> r.durPhase2
            >> r.durPhase3
            >> r.durPhase4
            >> r.durPhase5
            >> enumVal
            >> r.softSpill
            >> r.timerPeriod
            ;
    r.source = static_cast<SpillConfig::spill_sel_t>(enumVal);
    return in;
}
