#include "TdcCoreConfigConverter.h"
#include "base/ConfigConverterUtil.h"

#include "TdcCoreConfig.h"

namespace {
constexpr const char *k_autoResolution = "autoResolution";
constexpr const char *k_chTdcEn = "chTdcEn";
constexpr const char *k_latency = "latency";
constexpr const char *k_matchWin = "matchWin";

constexpr const char *k_hptdcHdr = "hptdcHdr";
constexpr const char *k_hptdcTrl = "hptdcTrl";
constexpr const char *k_leadEdge = "leadEdge";
constexpr const char *k_trailEdge = "trailEdge";
constexpr const char *k_adjust_set = "adjust_set";
} // namespace

template<>
bool AbstractConfigConverter<TdcCoreConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TdcCoreConfig>::toJsonObject(const TdcCoreConfig &in)
{
    QVariantMap m;

    //    m[k_chTdcEn] = QVariant::fromValue(in.chTdcEn);
    m[k_chTdcEn] = ConfigConverterUtil::toJsonValue<int, bool>(in.chTdcEn);
    m[k_latency] = QVariant::fromValue(in.latency);
    m[k_matchWin] = QVariant::fromValue(in.matchWin);

    m[k_hptdcHdr] = QVariant::fromValue(in.hptdcHdr);
    m[k_hptdcTrl] = QVariant::fromValue(in.hptdcTrl);
    m[k_leadEdge] = QVariant::fromValue(in.leadEdge);
    m[k_trailEdge] = QVariant::fromValue(in.trailEdge);
    m[k_autoResolution] = QVariant::fromValue(in.autoResolution);
    m[k_adjust_set] = QVariant::fromValue(in.adjustConfig);

    return fromVariantMap(m);
}

template<>
TdcCoreConfig AbstractConfigConverter<TdcCoreConfig>::fromJsonObject(const QJsonObject &in)
{
    TdcCoreConfig out;

    //    update_value(in, k_chTdcEn, &out.chTdcEn );
    out.chTdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chTdcEn]);
    update_value(in, k_latency, &out.latency);
    update_value(in, k_matchWin, &out.matchWin);

    update_value(in, k_hptdcHdr, &out.hptdcHdr);
    update_value(in, k_hptdcTrl, &out.hptdcTrl);
    update_value(in, k_leadEdge, &out.leadEdge);
    update_value(in, k_trailEdge, &out.trailEdge);
    update_value(in, k_autoResolution, &out.autoResolution);
    update_value(in, k_adjust_set, &out.adjustConfig);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<TdcCoreConfig>::toJsonValue(const TdcCoreConfig &in)
{
    return toJsonObject(in);
}

template<>
TdcCoreConfig AbstractConfigConverter<TdcCoreConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TdcCoreConfig>::registerSubTypes()
{
    AbstractConfigConverter<HptdcAdjustConfig>::registerTypes();
}
