#include "Tlu16SfpTriggerModule.h"

#include "DeviceModuleCommand.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID        = 0x00,
    REG_CTRL        = 0x02, // [0] - reset
    REG_LOCK        = 0x03, // [0] - 1: lock, 0: continuous update
    REG_NUM_CH      = 0x04, // Number of channels, RO static

    REG32_TOF_CH_EN = 0x10, // TOF channels enable, 32 bits, R/W
    REG_TOF_SUM_THR = 0x14, // TOF sum threshold, R/W
    REG32_LUM_CH_EN = 0x20, // Luminosity channels enable, 32 bits, R/W
};
}

QList<AFI_SDB_ID> Tlu16SfpTriggerModule::getSupportedSdbIds()
{
    return {AFI_SDB_TLU16SFP_Trigger};
}

bool Tlu16SfpTriggerModule::writeConfig(const Tlu16SfpTriggerConfig &config)
{
    if(!buildParams)
        return {};

    quint32 tofChEn = 0, lumChEn = 0;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        if(config.tofChEn.value(ch, false))
            tofChEn |= 1<<ch;
        if(config.lumChEn.value(ch, false))
            lumChEn |= 1<<ch;
    }

    RegOpVector r;
    r.RegWrite32(REG32_TOF_CH_EN, tofChEn);
    r.RegWrite16(REG_TOF_SUM_THR, config.tofSumThr);
    r.RegWrite32(REG32_LUM_CH_EN, lumChEn);
    return regOpExecRebased(r);
}

void Tlu16SfpTriggerModule::post_init()
{
    buildParams.reset();
    quint32 id;
    Tlu16SfpTriggerBuildParams bp;

    RegOpVector r;
    r.RegRead32(REG32_ID, &id);
    r.RegRead16(REG_NUM_CH, &bp.chNum);

    if(regOpExecRebased(r) && getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(id))) {
        buildParams = bp;
    }
}
