#include "TluDecDesModule.h"

#include "DeviceModuleCommand.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID                    = 0x00,
    REG_CTRL                    = 0x02, // [0] - reset
    REG_LOCK                    = 0x03, // [0] - 1: lock, 0: continuous update
    REG_NUM_CH                  = 0x04, // Number of channels, RO static

    REG_SETTINGS_BASE_ADDR      = 0x08, // Settings array address, RO static
    REG_STATUS_BASE_ADDR        = 0x09, // Statuses array address, RO static
    REG_BITSLIP_BASE_ADDR       = 0x0A, // Bitslip tap values array address, RO static
    REG_IDELAY_BASE_ADDR        = 0x0B, // IDelay tap values array address, RO static
    REG_RX_CODE_ERR_BASE_ADDR   = 0x0C, // RX code error counters array address, RO static
};
}

QList<AFI_SDB_ID> TluDecDesModule::getSupportedSdbIds()
{
    return {AFI_SDB_TLU_Dec_Des};
}

bool TluDecDesModule::writeConfig(const TluDecDesConfig &config)
{
    if(!buildParams)
        return {};

    RegOpVector r;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        auto addr = buildParams->settingsBaseAddr + ch;
        r.RegWrite16(addr, config.chReset.value(ch, TluDecDesConfig::DEF_CH_RESET) ? 1 : 0);
    }

    return regOpExecRebased(r);
}

std::optional<TluDecDesStatus> TluDecDesModule::readStatus()
{
    if(!buildParams)
        return {};

    TluDecDesStatus st;
    st.chSt.resize(buildParams->chNum);

    RegOpVector r;
    r.RegWrite16(REG_LOCK, 0x1);
    quint16 addr;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        auto &chSt = st.chSt[ch];

        addr = buildParams->statusBaseAddr + ch;
        r.RegRead16(addr, &chSt.status);

        addr = buildParams->bitslipBaseAddr + ch;
        r.RegRead16(addr, &chSt.bitSlip);

        addr = buildParams->idelayBaseAddr + ch;
        r.RegRead16(addr, &chSt.iDelay);

        addr = buildParams->rxCodeErrCntBaseAddr + ch*2;
        r.RegRead32(addr, &chSt.rxCodeErrCnt);
    }
    r.RegWrite16(REG_LOCK, 0x0);

    if(!regOpExecRebased(r))
        return {};
    return st;
}

void TluDecDesModule::post_init()
{
    buildParams.reset();
    quint32 id;
    TluDecDesBuildParams bp;

    RegOpVector r;
    r.RegRead32(REG32_ID, &id);
    r.RegRead16(REG_NUM_CH, &bp.chNum);

    r.RegRead16(REG_SETTINGS_BASE_ADDR, &bp.settingsBaseAddr);
    r.RegRead16(REG_STATUS_BASE_ADDR, &bp.statusBaseAddr);
    r.RegRead16(REG_BITSLIP_BASE_ADDR, &bp.bitslipBaseAddr);
    r.RegRead16(REG_IDELAY_BASE_ADDR, &bp.idelayBaseAddr);
    r.RegRead16(REG_RX_CODE_ERR_BASE_ADDR, &bp.rxCodeErrCntBaseAddr);

    if(regOpExecRebased(r) && getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(id))) {
        buildParams = bp;
    }
}
