#include "PhysicalTriggerConfigConverter.h"


#include "base/ConfigConverterUtil.h"

#include "PhysicalTriggerConfig.h"

namespace {
constexpr const char *k_name = "name";
constexpr const char *k_reg_all = "reg_all";
constexpr const char *k_reg = "reg";
constexpr const char *k_after_protect_time = "after_protect_time";
constexpr const char *k_before_protect_time = "before_protect_time";
constexpr const char *k_reduction_factor = "reduction_factor";
constexpr const char *k_delay = "delay";
constexpr const char *k_reduction_en = "reduction_en";
constexpr const char *k_after_protect_en = "after_protect_en";
constexpr const char *k_before_protect_en = "before_protect_en";

constexpr const char *k_en = "en";
constexpr const char *k_period = "period";
constexpr const char *k_count = "count";
constexpr const char *k_offset = "offset";

constexpr const char *k_matchWin = "matchWin";
constexpr const char *k_trigDelay = "trigDelay";
constexpr const char *k_l0Extend = "l0Extend";
constexpr const char *k_calibTrigger = "calibTrigger";
constexpr const char *k_startChMask = "startChMask";
constexpr const char *k_readChMask = "readChMask";
constexpr const char *k_chWfLatency = "chWfLatency";
constexpr const char *k_chWfReadPoints = "chWfReadPoints";
constexpr const char *k_oscEn = "oscEn";
constexpr const char *k_oscDelayed = "oscDelayed";
constexpr const char *k_chCfg = "chCfg";
constexpr const char *k_trig = "trig";
constexpr const char *k_spill = "spill";
}

template<>
bool AbstractConfigConverter<PhysicalTriggerChConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<PhysicalTriggerChConfig>::toJsonObject(const PhysicalTriggerChConfig &in)
{
    QVariantMap m;
    m[k_name] = QVariant::fromValue(in.name);

    m[k_reg_all] = jsonValueFromUInt64(in.reg.all);

    QVariantMap reg;
    reg[k_after_protect_time] = in.reg.bit.after_protect_time;
    reg[k_before_protect_time] = in.reg.bit.before_protect_time;
    reg[k_reduction_factor] = in.reg.bit.reduction_factor;
    reg[k_delay] = in.reg.bit.delay;
    reg[k_reduction_en] = in.reg.bit.reduction_en;
    reg[k_after_protect_en] = in.reg.bit.after_protect_en;
    reg[k_before_protect_en] = in.reg.bit.before_protect_en;
    m[k_reg] = reg;

    return fromVariantMap(m);
}

template<>
PhysicalTriggerChConfig AbstractConfigConverter<PhysicalTriggerChConfig>::fromJsonObject(const QJsonObject &in)
{
    PhysicalTriggerChConfig out;
    update_value(in, k_name, &out.name);

    if(in.contains(k_reg_all))
        out.reg.all = jsonValueToUInt64(in.value(k_reg_all));

    return out;
}

template<>
QJsonValue AbstractConfigConverter<PhysicalTriggerChConfig>::toJsonValue(const PhysicalTriggerChConfig &in)
{
    return toJsonObject(in);
}

template<>
PhysicalTriggerChConfig AbstractConfigConverter<PhysicalTriggerChConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<PhysicalTriggerChConfig>::registerSubTypes()
{
}

// =========================

template<>
bool AbstractConfigConverter<CalibrationTriggerConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<CalibrationTriggerConfig>::toJsonObject(const CalibrationTriggerConfig &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_period] = QVariant::fromValue(in.period);
    m[k_count] = QVariant::fromValue(in.count);
    m[k_offset] = QVariant::fromValue(in.offset);
    return fromVariantMap(m);
}

template<>
CalibrationTriggerConfig AbstractConfigConverter<CalibrationTriggerConfig>::fromJsonObject(const QJsonObject &in)
{
    CalibrationTriggerConfig out;
    update_value(in, k_en, &out.en);
    update_value(in, k_period, &out.period);
    update_value(in, k_count, &out.count);
    update_value(in, k_offset, &out.offset);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<CalibrationTriggerConfig>::toJsonValue(const CalibrationTriggerConfig &in)
{
    return toJsonObject(in);
}

template<>
CalibrationTriggerConfig AbstractConfigConverter<CalibrationTriggerConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<CalibrationTriggerConfig>::registerSubTypes()
{
}

// =========================

template<>
bool AbstractConfigConverter<PhysicalTriggerConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<PhysicalTriggerConfig>::toJsonObject(const PhysicalTriggerConfig &in)
{
    QVariantMap m;
    m[k_matchWin] = QVariant::fromValue(in.matchWin);
    m[k_trigDelay] = QVariant::fromValue(in.trigDelay);
    m[k_l0Extend] = QVariant::fromValue(in.l0Extend);
    m[k_calibTrigger] = QVariant::fromValue(in.calibTrigger);
    m[k_startChMask] = QVariant::fromValue(in.startChMask);
    m[k_readChMask] = QVariant::fromValue(in.readChMask);
    m[k_chWfLatency] = QVariant::fromValue(in.chWfLatency);
    m[k_chWfReadPoints] = QVariant::fromValue(in.chWfReadPoints);
    m[k_oscEn] = QVariant::fromValue(in.oscEn);
    m[k_oscDelayed] = QVariant::fromValue(in.oscDelayed);
    m[k_chCfg] = ConfigConverterUtil::toJsonValue(in.chCfg);
    m[k_trig] = QVariant::fromValue(in.trig);
    m[k_spill] = QVariant::fromValue(in.spill);

    return fromVariantMap(m);
}

template<>
PhysicalTriggerConfig AbstractConfigConverter<PhysicalTriggerConfig>::fromJsonObject(const QJsonObject &in)
{
    PhysicalTriggerConfig out;
    update_value(in, k_matchWin, &out.matchWin);
    update_value(in, k_trigDelay, &out.trigDelay);
    update_value(in, k_l0Extend, &out.l0Extend);
    update_value(in, k_calibTrigger, &out.calibTrigger);
    update_value(in, k_startChMask, &out.startChMask);
    update_value(in, k_readChMask, &out.readChMask);
    update_value(in, k_chWfLatency, &out.chWfLatency);
    update_value(in, k_chWfReadPoints, &out.chWfReadPoints);
    update_value(in, k_oscEn, &out.oscEn);
    update_value(in, k_oscDelayed, &out.oscDelayed);

    out.chCfg = ConfigConverterUtil::toVector<PhysicalTriggerChConfig>(in[k_chCfg]);
    update_value(in, k_trig, &out.trig);
    update_value(in, k_spill, &out.spill);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<PhysicalTriggerConfig>::toJsonValue(const PhysicalTriggerConfig &in)
{
    return toJsonObject(in);
}

template<>
PhysicalTriggerConfig AbstractConfigConverter<PhysicalTriggerConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<PhysicalTriggerConfig>::registerSubTypes()
{
    AbstractConfigConverter<PhysicalTriggerChConfig>::registerTypes();
    AbstractConfigConverter<CalibrationTriggerConfig>::registerTypes();
    AbstractConfigConverter<RcTrigConfig>::registerTypes();
    AbstractConfigConverter<SpillConfig>::registerTypes();
}
