#include "TrcErcModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG_CSR = 0,
    REG64_EVENT_NUM_LOAD = 4,
    REG64_EVENT_NUM_READ = 8,
};

enum {
    REG_TRIG_CSR_BIT_CNT_RESET = 1,
    REG_LOCK_BIT_LOCK = 2,
    REG_TRIG_CSR_BIT_OCSIL_EN = 4,
};
}

QList<AFI_SDB_ID> TrcErcModule::getSupportedSdbIds()
{
    return {AFI_SDB_TRC_ERC};
}

bool TrcErcModule::readGlobEvNum(quint64 *globEvNum)
{
    RegOpVector r;
    r.RegWrite16(REG_CSR, getControlReg() | REG_LOCK_BIT_LOCK);
    r.RegRead64(REG64_EVENT_NUM_READ, globEvNum);
    r.RegWrite16(REG_CSR, getControlReg());
    return regOpExecRebased(r);
}

std::optional<quint64> TrcErcModule::readGlobEvNum()
{
    quint64 globEvNum;
    if (!readGlobEvNum(&globEvNum))
        return {};
    return globEvNum;
}

bool TrcErcModule::writeConfig(bool oscEn)
{
    channelOscilogramEnable = oscEn;
    RegOpVector r;
    r.RegWrite16(REG_CSR, getControlReg());
    return regOpExecRebased(r);
}

RegOpVector TrcErcModule::setGlobEvNumRegOp(quint64 evNum) const
{
    RegOpVector r;
    r.RegWrite64(REG64_EVENT_NUM_LOAD, evNum);
    r.RegWrite16(REG_CSR, getControlReg()
                         | REG_TRIG_CSR_BIT_CNT_RESET); // reset stats
    r.RegWrite16(REG_CSR, getControlReg());
    return r;
}

bool TrcErcModule::setGlobEvNum(quint64 evNum)
{
    auto r = setGlobEvNumRegOp(evNum);
    return regOpExecRebased(r);
}

quint16 TrcErcModule::getControlReg() const
{
    return channelOscilogramEnable ? REG_TRIG_CSR_BIT_OCSIL_EN : 0;
}
