#include "Ddr3StatusTableHelper.h"

#include <QTableWidget>

#include "device-modules/Ddr3StreamFifo/Ddr3StreamFifoStatus.h"

void Ddr3StatusTableHelper::setHeaders(QTableWidget *table, int colOffset)
{
    table->setHorizontalHeaderItem(colOffset+COL_STATUS, new QTableWidgetItem("DDR3 St."));
    table->setHorizontalHeaderItem(colOffset+COL_OCCUPANCY, new QTableWidgetItem("DDR3 occup."));
    table->setHorizontalHeaderItem(colOffset+COL_ECC_MULTI_ERR, new QTableWidgetItem("ECC multi"));
    table->setHorizontalHeaderItem(colOffset+COL_ECC_SINGLE_ERR, new QTableWidgetItem("ECC single"));
}

void Ddr3StatusTableHelper::updateDevStatus(QTableWidget *table, int colOffset, int row, const Ddr3StreamFifoStatus &st)
{
    const auto itemStatus = table->item(row, colOffset+COL_STATUS);
    const auto itemOccup = table->item(row, colOffset+COL_OCCUPANCY);
    const auto itemEccMulti = table->item(row, colOffset+COL_ECC_MULTI_ERR);
    const auto itemEccSingle = table->item(row, colOffset+COL_ECC_SINGLE_ERR);

    QString statusStr;
    auto stBits = st.statusCode.bit;
    if(stBits.initCalComplete) {
        if(stBits.empty)
            statusStr = "empty";
        else if (stBits.full)
            statusStr = "full";
        else
            statusStr = "Ok";
    } else {
        statusStr = "Init/Calib failed";
    }

    const QColor bgColorDefault = table->style()->standardPalette().light().color();
    const QColor bgColorWarning = Qt::yellow;
    const QColor bgColorError = Qt::red;

    itemStatus->setText(statusStr);
    double occ = 100. * st.fifoOccup/st.fullMem();
    itemOccup->setText(st.fullMem() ? QString("%1%").arg(occ,0,'f',1) : QString("-"));
    itemOccup->setToolTip(QString("occ: %L1 Bytes\nfull: %L2 Bytes").arg(st.fifoOccup).arg(st.fullMem()));
    itemEccMulti->setText(QString("%L1").arg(st.eccMulti));
    itemEccSingle->setText(QString("%L1").arg(st.eccSingle));

    itemEccMulti->setBackground(st.eccMulti ? bgColorError : bgColorDefault);
    itemEccSingle->setBackground(st.eccSingle ? bgColorWarning : bgColorDefault);
}
