/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "WaveBlcWidget.h"
#include "base/gui_utils.h"
#include "base/gui_utils.h"
#include "ui_WaveBlcWidget.h"

#include <QSettings>


WaveBlcWidget::WaveBlcWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WaveBlcWidget)
{
    ui->setupUi(this);
    ui->labelTruncate->hide();
    ui->spinBoxTruncate->hide();

    RecursiveWidgetSignalBlocker blocker(this);
    ui->comboBoxPolarity->addItem("Positive", WaveBlcChannelConfig::Positive);
    ui->comboBoxPolarity->addItem("Negative", WaveBlcChannelConfig::Negative);

    ui->comboBoxMafLength->addItem("1", 1);
    ui->comboBoxMafLength->addItem("2", 2);
    ui->comboBoxMafLength->addItem("4", 4);
    ui->comboBoxMafLength->addItem("8", 8);
    ui->comboBoxMafLength->addItem("16", 16);

    ui->comboBoxMode->addItem("Bypass", WaveBlcChannelConfig::Bypass);
    ui->comboBoxMode->addItem("Correction", WaveBlcChannelConfig::Correction);
    ui->comboBoxMode->addItem("Debug", WaveBlcChannelConfig::Debug);
    ui->comboBoxMode->addItem("MAF output", WaveBlcChannelConfig::Test);

    {
        auto range = WaveBlcChannelConfig::thrRange();
        ui->spinBoxThreshold->setRange(range.first, range.second);
        ui->spinBoxThreshold->setSingleStep(4);
    }
    {
        auto range = WaveBlcChannelConfig::mafRoundoffRange();
        ui->spinBoxTruncate->setRange(range.first, range.second);
    }
    ui->spinBoxCommonOffset->setRange(std::numeric_limits<qint16>::min(),
                                std::numeric_limits<qint16>::max());
    restoreGuiFromConfig();
}

WaveBlcWidget::~WaveBlcWidget()
{
    delete ui;
}

void WaveBlcWidget::setWaveBlcConfig(const WaveBlcConfig &newConfig)
{
    if (config == newConfig)
        return;
    config = newConfig;

    restoreGuiFromConfig();
}

void WaveBlcWidget::restoreGuiFromConfig()
{
    RecursiveWidgetSignalBlocker blocker(this);

    setComboBoxCurrentItem(ui->comboBoxPolarity, (int)config.common.polarity);
    setComboBoxCurrentItem(ui->comboBoxMode, (int)config.common.mode);
    setComboBoxCurrentItem(ui->comboBoxMafLength, config.common.mafLength);

    ui->spinBoxThreshold->setValue(config.common.thr);
    ui->spinBoxTruncate->setValue(config.common.mafRoundoff);
    ui->spinBoxCommonOffset->setValue(config.commonOffset);
}

void WaveBlcWidget::on_spinBoxThreshold_valueChanged(int arg1)
{
    config.common.thr = arg1;
    emit waveBlcConfigChanged(config);
}

void WaveBlcWidget::on_spinBoxTruncate_valueChanged(int arg1)
{
    config.common.mafRoundoff = arg1;
    emit waveBlcConfigChanged(config);
}

void WaveBlcWidget::on_comboBoxPolarity_currentIndexChanged(int index)
{
    config.common.polarity = static_cast<WaveBlcChannelConfig::Polarity>(ui->comboBoxPolarity->itemData(index).toInt());
    emit waveBlcConfigChanged(config);
}


void WaveBlcWidget::on_comboBoxMode_currentIndexChanged(int index)
{
    config.common.mode = static_cast<WaveBlcChannelConfig::Mode>(ui->comboBoxMode->itemData(index).toInt());
    emit waveBlcConfigChanged(config);
}


void WaveBlcWidget::on_comboBoxMafLength_currentIndexChanged(int index)
{
    config.common.mafLength = ui->comboBoxMafLength->itemData(index).toInt();
    emit waveBlcConfigChanged(config);
}

void WaveBlcWidget::on_spinBoxCommonOffset_valueChanged(int arg1)
{
    config.commonOffset = arg1;
    emit waveBlcConfigChanged(config);
}
