#ifndef BASESIGNALROUTER_H
#define BASESIGNALROUTER_H

#include <QObject>

class ConfigurationManager;
class BaseFsm;
class BaseAppCore;
class PnpBaseMainWindow;
class RemoteControlServer;

class BaseSignalRouter
{
public:
    explicit BaseSignalRouter() = default;
    virtual ~BaseSignalRouter() = default;

    void setConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }
    void setMainFsm(BaseFsm *p)
    { mainFsm = p; }
    void setBaseAppCore(BaseAppCore *p);
    void setPnpBaseMainWindow(PnpBaseMainWindow *p);
    void setRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }

public:
    virtual void connectAll() const;
    virtual void connect_FSM() const;
    virtual void connect_MainWindow() const;
    virtual void connect_PnpServer() const;
    virtual void connect_RemoteControlServer() const;

    void removeRemoteControlServer();

protected:
    template<typename Func1, typename Func2>
    static inline void connect
        (const typename QtPrivate::FunctionPointer<Func1>::Object *sender,
         Func1 signal,
         const typename QtPrivate::FunctionPointer<Func2>::Object *receiver,
         Func2 slot)
    {
        QObject::connect(sender, signal, receiver, slot);
    }
    template<typename Func1, typename Func2>
    static inline void connectUnique
        (const typename QtPrivate::FunctionPointer<Func1>::Object *sender,
         Func1 signal,
         const typename QtPrivate::FunctionPointer<Func2>::Object *receiver,
         Func2 slot)
    {
        QObject::connect(sender, signal, receiver, slot, Qt::UniqueConnection);
    }
    template <typename Func1>
    static inline bool disconnect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal)
    {
        return QObject::disconnect(sender, signal, nullptr, nullptr);
    }

    BaseFsm * mainFsm = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
    BaseAppCore *baseAppCore = nullptr;

#ifdef QT_WIDGETS_LIB
    ConfigurationManager *configurationManager = nullptr;
    PnpBaseMainWindow *pnpBaseMainWindow = nullptr;
#endif
};

#endif // BASESIGNALROUTER_H
