/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ReadoutAppCore.h"

#include <QDebug>
#include <QLoggingCategory>

#include "AsyncDeviceCommander.h"
#include "fsm/BaseFsm.h"
#include "DeviceStatusZmqPublisher/DeviceStatusZmqPublisher.h"
#include "remote-control-server/EvNumChecker.h"
#include "RunLogic/RunLogicController.h"
#include "mldiscover/DeviceDispatcher.h"

namespace {
Q_DECLARE_LOGGING_CATEGORY(lc_appcore)
Q_LOGGING_CATEGORY(lc_appcore, "afi.appcore")
}

ReadoutAppCore::ReadoutAppCore(QObject *parent)
    : BaseAppCore{parent}
    , deviceCommander(new AsyncDeviceCommander(this))
    , deviceDispatcher(new DeviceDispatcher(this))
    , deviceStatusZmqPublisher(new DeviceStatusZmqPublisher(this))
    , evNumChecker(new EvNumChecker(this))
    , runLogicController(new RunLogicController(this))
{
}

ReadoutAppCore::~ReadoutAppCore()
{}

void ReadoutAppCore::deviceDisconnected(const DeviceIndex &index)
{
    if (isRun()) {
        emit sendRunError(rcRunId, index.getIdent() + " disconnected");
    }
}

void ReadoutAppCore::onEvNumCheckFail(const QString &mess) const
{
    if(!rcRunId.isValid())
        return;
    emit sendRunWarning(rcRunId, mess);
}

void ReadoutAppCore::requestAdcClkDivSync() const
{
    if (!isRun())
        emit adcClkDivSync();
}

bool ReadoutAppCore::isRun() const
{
    auto st = getFsm()->getState();
    return st==FsmState::Run
        || st==FsmState::Starting
        || st==FsmState::Configure;
}
