#ifndef CLIENTMGRFSM_H
#define CLIENTMGRFSM_H

#include <QMap>
#include <QSet>

#include "base/ClientIndex.h"
#include "fsm/BaseFsm.h"

struct ClientsFsmInput : public BaseFsmInput
{
    ClientIndexSet clientSet;
    QMap<ClientIndex, FsmState> clientStates;
};

class ClientManagerFsm : public BaseFsm
{
    Q_OBJECT
public:
    explicit ClientManagerFsm(QObject *parent = nullptr);
    virtual bool isTransitionAllowed(const FsmState &toState) const override;

public slots:
    void setClientList(const ClientIndexSet &newList,
                       const QMap<ClientIndex, FsmState> &clientStates);
    void setClientState(const ClientIndex &id, const FsmState &clState);

protected slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline ClientsFsmInput *getInput() {
        return static_cast<ClientsFsmInput*>(BaseFsm::getInput());
    }
    inline const ClientsFsmInput *getInput() const {
        return static_cast<const ClientsFsmInput*>(BaseFsm::getInput());
    }

private:
    bool allClientsInState(const FsmState &state) const;
    bool isAnyClientInError() const;
};

QDebug operator<<(QDebug dbg, const ClientsFsmInput &state);

#endif // CLIENTMGRFSM_H
