/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCCLIENTDATA_H
#define RCCLIENTDATA_H

#include "config/RcClientProtoVersions.h"
#include "fsm/FsmState.h"
#include "RunStates.h"

#include "base/ClientIndex.h"

#include <QElapsedTimer>
#include <QHostAddress>
#include <QMetaType>
#include <QString>
#include <QUuid>

struct ClientWarn {
    friend bool operator==(const ClientWarn &, const ClientWarn &) = default;

    static const qint64 TIMEOUT = 10e3; // in ms

    bool active = false;
    QString msg;
    quint32 spillNum = 0;
    QElapsedTimer timer;
    bool isActive(quint32 curSpillNum) const {
        if(!active || timer.elapsed() > TIMEOUT)
            return false;
        return spillNum == curSpillNum || spillNum+1 == curSpillNum;
    }
};

struct RcClientPeerInfo {
    friend bool operator==(const RcClientPeerInfo &, const RcClientPeerInfo &) = default;
    QHostAddress host;
    quint16 port = 0;
    QString hostName;
};
Q_DECLARE_METATYPE(RcClientPeerInfo)

struct ClientInfo {
    ClientInfo(ClientIndex i = ClientIndex()) : index(i) {}
    friend bool operator==(const ClientInfo &, const ClientInfo &) = default;

    inline ClientIndex getIndex() const { return index; }
    bool hasVme() const {
        return has_vme ||
                index.getProgramType() == DaqModule(DaqModule::Type::VmeDaq);
    }

private:
    ClientIndex index;

public:
    RcClientPeerInfo clientStatus;
    RC_CLIENT_PROTO_ID protoId = RC_CLIENT_PROTO_UNKNOWN;
    FsmState fsmState = FsmState::Unknown;
    QString status;
    ClientWarn warn;
    bool has_vme = false;
    bool backPressure = false;
    QUuid prog_uuid;
};
Q_DECLARE_METATYPE(ClientInfo)

#endif // RCCLIENTDATA_H
