/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MlinkStateFsm.h"

#include <QDebug>

bool MlinkStateFsm::changeState(State newState)
{
    static const QVector<QPair<State, State>> map {
        // {oldState, newState}, reason
        {Closed, Detected},    // detect() true
        {Closed, Failed},      // detect() retry_count > limit
        {Detected, Connected}, // checkLiveMagic() true
        {Detected, Failed},    // checkLiveMagic() I/O error
        {Connected, Closed},   // forceDisconnect()
        {Connected, Failed},   // checkLiveMagic() false [device reboot]
        {Failed, Closed}      // recover ?
    };
    bool transitionAllowed = map.contains({curState, newState});
    Q_ASSERT(transitionAllowed);
    if (!transitionAllowed)
        return false;
    auto oldState = curState;
    curState = newState;
    stateTimer.restart();
    emit stateChanged(oldState, newState);
    return true;
}

QString MlinkStateFsm::stateStr(State s)
{
    switch(s) {
    case MlinkStateFsm::Closed:
        return "Closed";
    case MlinkStateFsm::Detected:
        return "Detected";
    case MlinkStateFsm::Connected:
        return "Connected";
    case MlinkStateFsm::Failed:
        return "Failed";
    }
    return {};
}

QDebug operator<<(QDebug dbg, MlinkStateFsm::State state)
{
    dbg << MlinkStateFsm::stateStr(state);
    return dbg;
}
