﻿#ifndef CONFIGURATIONMANAGER_H
#define CONFIGURATIONMANAGER_H

#include <QDialog>

#include "base/ConfigIndexName.h"
#include "base/DaqModule.h"
#include "mongo/BaseConfig.h"

class JsonTreeModel;
class ConfigListModel;

namespace Ui {
class ConfigurationManager;
}

class ConfigurationManager : public QDialog
{
    Q_OBJECT
public:
    explicit ConfigurationManager(QWidget *parent = nullptr);
    ~ConfigurationManager() override;
    virtual void setVisible(bool visible) override;

    template<class T>
    void setDefaultConfig(const T &obj)
    {
        default_json = QVariant::fromValue(obj).template value<QJsonObject>();
    }

signals:
    void switch_to_config(const ConfigIndexName &conf) const;

public slots:
    void configSelectorChanged(const ConfigIndexName &cin);

private slots:
    void on_pushButton_new_clicked();
    void on_pushButton_rename_clicked();
    void on_pushButton_clone_clicked();
    void on_pushButton_delete_clicked();
    void on_pushButtonRefresh_clicked();
    void on_pushButton_switch_to_clicked();
    void on_treeViewCfgList_doubleClicked(const QModelIndex &index);

    void currentCfgIndexChanged(const QModelIndex &current);

    void on_checkBox_stateChanged(int arg1);

private:
    void reset();
    void exec_sub_dialog(int type);
    void populate_config_list_tree();
    void update_label_db_status();

    Ui::ConfigurationManager *ui;
    bool initialized = false;
    BaseConfig base_config;

    const DaqModule program_type;
    ConfigIndexName configuration;
    ConfigIndexName configuration_selected;

    QJsonObject default_json;
    QJsonObject selected_json;
    QMap<ProgramIndex, QMap<ConfigurationName, QDateTime> > config_map;
    JsonTreeModel * const model_config_info;
    ConfigListModel * const model_configs_list;
};

#endif // CONFIGURATIONMANAGER_H
