﻿/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MONGOQT_H
#define MONGOQT_H

#include <QObject>
#include <QString>
#include <QJsonObject>
#include <QTimer>

// temporary includes
#ifndef Q_MOC_RUN
#include <mongocxx/pool.hpp>
#include <bsoncxx/builder/basic/document.hpp>
#endif

#include "MongoPipeline.h"

class MongoQtPrivate;
class MongoQt
{
public:
    friend class TestMongoQt;
    MongoQt();
    virtual ~MongoQt();
    void set_db(const QString &db_) { db = db_; }
    void set_collection(const QString &name_) { collection_name = name_; }
    void set_db_collection(const QString &db_, const QString &collection_name_);
    QString get_db(){return db;}
    QString get_collection_name(){return collection_name;}
    bool set_uri(const QString &uri);
    int count_documents();
    QList<QJsonObject> find(
        const QJsonObject &query = QJsonObject(),
        const QJsonObject &projection = QJsonObject());
    bool replace_one(
        const QJsonObject &filter,
        const QJsonObject &replacement,
        bool upsert = true);
    bool insert_one(const QJsonObject &obj);
    bool delete_one(const QJsonObject &obj);
    int delete_many(const QJsonObject &obj);
    int delete_all();

    // temporary proxies (TODO: use pimpl)
    mongocxx::pool::entry get_connection();
    mongocxx::collection get_collection();
public slots:
    bool try_connection();

private:
    Q_DISABLE_COPY(MongoQt)
    Q_DECLARE_PRIVATE(MongoQt)
    MongoQtPrivate * const d_ptr;
    QString db;
    QString collection_name;
};

#endif // MONGOQT_H
