#ifndef ADDPNPDIALOG_H
#define ADDPNPDIALOG_H

#include <QDialog>

#include "AddPnpComboBoxFilter.h"
#include "pnp-server/PNPMessage.h"

class ProgramDescription;
class QTableView;
class QTableWidgetItem;
class QComboBox;
class QPushButton;
class AddPnpModel;
class QItemSelection;

namespace Ui {
class AddClientDialog;
}

class AddPnpDialog : public QDialog
{
    Q_OBJECT

public:
    explicit AddPnpDialog(PnpInfoCache progDescrs,
                          bool multi_select = true,
                          QWidget *parent = nullptr);
    ~AddPnpDialog();
    QVector<ProgramDescription> getProgDescr() const;
    void set_function_filter(std::function<bool (const ProgramDescription &)> f);
    void set_selected_progs(PnpInfoCache progDescrs);

public slots:
    void pnpCacheUpdated(const PnpInfoCache &cache);

private slots:
    void on_comboBoxType_currentIndexChanged(int i);
    void on_comboBoxHost_currentIndexChanged(int i);
    void on_comboBoxIndex_currentIndexChanged(int i);
    void on_leftTableView_doubleClicked(const QModelIndex &index);
    void on_rightTableView_doubleClicked(const QModelIndex &index);
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();

    void tableView_selectionChanged(const QItemSelection &selected, const QItemSelection &selected2);
    void model_reseted();

private:
    void applyProgFilter();
    void populate_all_filter();
    QVariant getComboBoxField(QComboBox *cb, const ProgramDescription &pd) const;
    void populate_program_filter(QComboBox *box);
    void filter_item_changed();
    void update_detail_label(const ProgramDescription *row_struct);
    void set_current_index(QComboBox *box, const QVariant &item);
    void init_table(QTableView *table, AddPnpModel *model);
    void refreshTableSelection(bool isLeft);
    void set_push_button_enable(bool isAdd, bool enable);
    void change_button_color(QPushButton *push_button, QColor color);
    const ProgramDescription *modelIndex_to_progdescr(const QModelIndex &i) const;
    void check_add_remove_button_enable(bool isAdd);
    void check_ok_button_enable();
    bool is_prog_match_cb_filter(const ProgramDescription &pd) const;
    bool is_prog_match_cb_filter(const ProgramDescription &pd,
                              const AddPnpComboBoxFilter &filter) const;
    void add_prog(const ProgramDescription *pdPtr);
    void remove_prog(const ProgramDescription *pdPtr);

    Ui::AddClientDialog *ui;
    PnpInfoCache pnpCache;
    PnpInfoCache filtredProgs; // pnpCache after filter_func
    PnpInfoCache selected_progs;
    const bool multi_select;
    AddPnpComboBoxFilter cbFilter;
    std::function<bool(const ProgramDescription&)> filter_func = nullptr;
    QUuid detailed_program_uuid;
    AddPnpModel * const leftModel;
    AddPnpModel * const rightModel;
};

#endif // ADDPNPDIALOG_H
